﻿#region

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using JetBrains.Annotations;
using Styx;
using Styx.Common;
using Styx.CommonBot;
using Styx.Helpers;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;

#endregion

namespace McLovin
{
    internal static class Spell
    {
        public static readonly Dictionary<string, DoubleCastSpell> DoubleCastEntries =
            new Dictionary<string, DoubleCastSpell>();

        public static string lastlog = "";

        internal static WoWAuraCollection CachedGetAllAuras(this WoWUnit unit, int expiry = 1000)
        {
            var cacheKey = "GetAllAuras_" + unit.Guid;

            var getAllAuras = CacheManager.Get<WoWAuraCollection>(cacheKey);

            if (getAllAuras == null)
            {
                getAllAuras = unit.GetAllAuras();
                CacheManager.Add(getAllAuras, cacheKey, expiry);
            }
            return getAllAuras;
        }

        public static bool CachedHasAura(this WoWUnit unit, int aura, int stacks = 0, bool fromMyAura = false,
                                         int msLeft = 0, int expiry = 1000)
        {
            var cachedResult = unit.CachedGetAllAuras(expiry).FirstOrDefault(a => a.SpellId == aura);

            if (cachedResult == null)
                return false;

            if (fromMyAura && cachedResult.CreatorGuid != StyxWoW.Me.Guid)
                return false;

            if (cachedResult.TimeLeft.TotalMilliseconds > msLeft)
                return cachedResult.StackCount >= stacks;

            return false;
        }

        public static bool CachedHasAura(this WoWUnit unit, string aura, int stacks = 0, bool fromMyAura = false,
                                         int msLeft = 0, int expiry = 1000)
        {
            var cachedResult = unit.CachedGetAllAuras(expiry).FirstOrDefault(a => a.Name == aura);

            if (cachedResult == null)
                return false;

            if (fromMyAura && cachedResult.CreatorGuid != StyxWoW.Me.Guid)
                return false;

            if (cachedResult.TimeLeft.TotalMilliseconds > msLeft)
                return cachedResult.StackCount >= stacks;

            return false;
        }

        public static uint CachedStackCount(this WoWUnit unit, int aura, bool fromMyAura = false, int expiry = 1000)
        {
            {
                if (unit != null)
                {
                    var cachedResult = unit.CachedGetAllAuras(expiry).FirstOrDefault(a => a.SpellId == aura);

                    if (cachedResult == null)
                        return 0;

                    if (fromMyAura && cachedResult.CreatorGuid != StyxWoW.Me.Guid)
                        return 0;

                    return cachedResult.StackCount;
                }
                return 0;
            }
        }

        public static double CachedGetAuraTimeLeft(this WoWUnit unit, int aura, bool fromMyAura = false,
                                                   int expiry = 1000)
        {
            if (unit != null)
            {
                var cachedResult = unit.CachedGetAllAuras(expiry).FirstOrDefault(a => a.SpellId == aura);

                if (cachedResult == null)
                    return 0;

                if (fromMyAura && cachedResult.CreatorGuid != StyxWoW.Me.Guid)
                    return 0;

                if (cachedResult.TimeLeft.TotalMilliseconds > 0)
                    return cachedResult.TimeLeft.TotalMilliseconds;
            }
            return 0;
        }

        public static double CachedGetAuraTimeLeft(this WoWUnit unit, string aura, bool fromMyAura = false,
                                                   int expiry = 1000)
        {
            if (unit != null)
            {
                var cachedResult = unit.CachedGetAllAuras(expiry).FirstOrDefault(a => a.Name == aura);

                if (cachedResult == null)
                    return 0;

                if (fromMyAura && cachedResult.CreatorGuid != StyxWoW.Me.Guid)
                    return 0;

                if (cachedResult.TimeLeft.TotalMilliseconds > 0)
                    return cachedResult.TimeLeft.TotalMilliseconds;
            }
            return 0;
        }

        public static double GetAuraTimeLeft(int aura)
        {
            return GetAuraTimeLeft(aura, StyxWoW.Me);
        }

        public static double GetAuraTimeLeft(int aura, WoWUnit onUnit)
        {
            if (onUnit != null)
            {
                var result = onUnit.GetAuraById(aura);
                if (result != null)
                {
                    if (result.TimeLeft.TotalSeconds > 0)
                        return result.TimeLeft.TotalSeconds;
                }
            }
            return 0;
        }

        public static double GetMyAuraTimeLeft(string aura, WoWUnit onUnit)
        {
            if (onUnit != null)
            {
                var result =
                    onUnit.GetAllAuras().FirstOrDefault(a => a.Name == aura && a.CreatorGuid == StyxWoW.Me.Guid);
                if (result != null && result.TimeLeft.TotalSeconds > 0)
                    return result.TimeLeft.TotalSeconds;
            }
            return 0;
        }

        public static bool HasMyAura(this WoWUnit unit, string aura)
        {
            return unit.GetAllAuras().Any(a => a.Name == aura && a.CreatorGuid == StyxWoW.Me.Guid);
        }

        public static TimeSpan GetSpellCooldown(string spell)
        {
            SpellFindResults results;
            if (SpellManager.FindSpell(spell, out results))
            {
                return results.Override != null ? results.Override.CooldownTimeLeft : results.Original.CooldownTimeLeft;
            }

            return TimeSpan.MaxValue;
        }

        public static TimeSpan GetSpellCooldown(int spell)
        {
            SpellFindResults results;
            if (SpellManager.FindSpell(spell, out results))
            {
                return results.Override != null ? results.Override.CooldownTimeLeft : results.Original.CooldownTimeLeft;
            }

            return TimeSpan.MaxValue;
        }

        public static uint CachedStackCount(this WoWUnit unit, string aura, bool fromMyAura = false, int expiry = 1000)
        {
            {
                if (unit != null)
                {
                    var cachedResult = unit.CachedGetAllAuras(expiry).FirstOrDefault(a => a.Name == aura);

                    if (cachedResult == null)
                        return 0;

                    if (fromMyAura && cachedResult.CreatorGuid != StyxWoW.Me.Guid)
                        return 0;

                    return cachedResult.StackCount;
                }
                return 0;
            }
        }

        public static bool HasAllAuras(this WoWUnit unit, params string[] auraNames)
        {
            return auraNames.All(unit.HasAura);
        }

        public static bool HasAnyAura(this WoWUnit unit, params string[] auraNames)
        {
            return auraNames.Any(unit.HasAura);
        }

        public static void UpdateDoubleCastEntries(string spellName, double expiryTime)
        {
            if (DoubleCastEntries.ContainsKey(spellName))
                DoubleCastEntries[spellName] = new DoubleCastSpell(spellName, expiryTime, DateTime.UtcNow);
            if (!DoubleCastEntries.ContainsKey(spellName))
                DoubleCastEntries.Add(spellName, new DoubleCastSpell(spellName, expiryTime, DateTime.UtcNow));
        }

        internal static void PulseDoubleCastEntries()
        {
            DoubleCastEntries.RemoveAll(
                t => DateTime.UtcNow.Subtract(t.DoubleCastCurrentTime).TotalSeconds >= t.DoubleCastExpiryTime);
        }

        public static bool SpellOnCooldown(int spell)
        {
            SpellFindResults results;
            if (SpellManager.FindSpell(spell, out results))
            {
                return results.Override != null ? results.Override.Cooldown : results.Original.Cooldown;
            }

            return false;
        }

        public static bool SpellOnCooldown(string spell)
        {
            SpellFindResults results;
            if (SpellManager.FindSpell(spell, out results))
            {
                return results.Override != null ? results.Override.Cooldown : results.Original.Cooldown;
            }
            return false;
        }

        /*CASTING STUFF */

        public static bool Heal(string spell, bool cd)
        {
            return !SpellOnCooldown(spell) && Heal(spell, StyxWoW.Me);
        }

        public static bool Heal(int spell, bool cd)
        {
            return !SpellOnCooldown(spell) && Heal(spell, StyxWoW.Me);
        }

        public static bool Heal(string spell)
        {
            return Heal(spell, StyxWoW.Me);
        }

        public static bool Heal(int spell, WoWUnit onUnit)
        {
            if (SpellManager.CanCast(spell, onUnit))
            {
                SpellManager.Cast(spell, onUnit);
                var clr = Color.Green;
                LogAction(WoWSpell.FromId(spell).Name, clr);
                return true;
            }
            return false;
        }

        public static bool Heal(string spell, WoWUnit onUnit)
        {
            if (SpellManager.CanCast(spell, onUnit))
            {
                SpellManager.Cast(spell, onUnit);
                var clr = Color.Green;
                LogAction(spell, clr);
                return true;
            }
            return false;
        }

        public static bool CastHack(int spell)
        {
            SpellManager.Cast(spell, StyxWoW.Me);
            var clr = Color.Yellow;
            LogAction(WoWSpell.FromId(spell).Name, clr);
            return true;
        }


        public static bool CastTalent(int spell)
        {
            return (GetSpellCooldown(spell).TotalSeconds - SpellManager.GlobalCooldownLeft.TotalSeconds < 0.1) &&
                   CastTalent(spell, StyxWoW.Me.CurrentTarget);
        }

        public static bool CastTalent(string spell)
        {
            return (GetSpellCooldown(spell).TotalSeconds - SpellManager.GlobalCooldownLeft.TotalSeconds < 0.1) &&
                   CastTalent(spell, StyxWoW.Me.CurrentTarget);
        }

        public static bool CastTalent(int spell, WoWUnit onUnit)
        {
            if (onUnit != null)
            {
                SpellManager.Cast(spell, onUnit);
                var clr = Color.Red;
                LogAction(WoWSpell.FromId(spell).Name, clr);
                return true;
            }
            return false;
        }

        public static bool CastTalent(string spell, WoWUnit onUnit)
        {
            if (onUnit != null)
            {
                SpellManager.Cast(spell, onUnit);
                var clr = Color.Red;
                LogAction(spell, clr);
                return true;
            }
            return false;
        }

        public static bool CastTalentBuff(string spell)
        {
            return (GetSpellCooldown(spell).TotalSeconds - SpellManager.GlobalCooldownLeft.TotalSeconds < 0.1) &&
                   CastTalent(spell, StyxWoW.Me);
        }

        public static bool CastTalentBuff(int spell)
        {
            return (GetSpellCooldown(spell).TotalSeconds - SpellManager.GlobalCooldownLeft.TotalSeconds < 0.1) &&
                   CastTalent(spell, StyxWoW.Me);
        }

        public static bool Cast(string spell)
        {
            return Cast(spell, StyxWoW.Me.CurrentTarget);
        }

        public static bool Cast(int spell)
        {
            return Cast(spell, StyxWoW.Me.CurrentTarget);
        }

        public static bool Cast(string spell, bool cd)
        {
            return !SpellOnCooldown(spell) && Cast(spell, StyxWoW.Me.CurrentTarget);
        }

        public static bool Cast(int spell, bool cd)
        {
            return !SpellOnCooldown(spell) && Cast(spell, StyxWoW.Me.CurrentTarget);
        }

        public static bool Cast(string spell, WoWUnit onUnit)
        {
            if (onUnit != null && SpellManager.CanCast(spell, onUnit))
            {
                SpellManager.Cast(spell, onUnit);
                var clr = Color.Red;
                LogAction(spell, clr);
                return true;
            }
            return false;
        }

        public static bool Cast(string spell, bool CD, WoWUnit onUnit)
        {
            if (SpellOnCooldown(spell))
                return false;
            if (onUnit != null && SpellManager.CanCast(spell, onUnit))
            {
                SpellManager.Cast(spell, onUnit);
                var clr = Color.Red;
                LogAction(spell, clr);
                return true;
            }
            return false;
        }

        public static bool Cast(int spell, bool CD, WoWUnit onUnit)
        {
            if (SpellOnCooldown(spell))
                return false;
            if (onUnit != null && SpellManager.CanCast(spell, onUnit))
            {
                SpellManager.Cast(spell, onUnit);
                var clr = Color.Red;
                LogAction(WoWSpell.FromId(spell).Name, clr);
                return true;
            }
            return false;
        }

        public static bool Cast(int spell, WoWUnit onUnit)
        {
            if (onUnit != null && SpellManager.CanCast(spell, onUnit))
            {
                SpellManager.Cast(spell, onUnit);
                var clr = Color.Red;
                LogAction(WoWSpell.FromId(spell).Name, clr);
                return true;
            }
            return false;
        }

        public static bool Buff(string spell, bool cd)
        {
            return !SpellOnCooldown(spell) && Buff(spell, StyxWoW.Me);
        }

        public static bool Buff(int spell, bool cd)
        {
            return !SpellOnCooldown(spell) && Buff(spell, StyxWoW.Me);
        }

        public static bool Buff(string spell, WoWUnit onUnit)
        {
            if (SpellManager.CanCast(spell, onUnit))
            {
                SpellManager.Cast(spell, onUnit);
                var clr = Color.Yellow;
                LogAction(spell, clr);
                return true;
            }
            return false;
        }

        public static bool Buff(int spell, WoWUnit onUnit)
        {
            if (SpellManager.CanCast(spell, onUnit))
            {
                SpellManager.Cast(spell, onUnit);
                var clr = Color.Yellow;
                LogAction(WoWSpell.FromId(spell).Name, clr);
                return true;
            }
            return false;
        }

        public static void LogAction(string log, Color colour)
        {
            var clr = colour;
            if (log == lastlog) return;
            System.Windows.Media.Color newColor = System.Windows.Media.Color.FromArgb(clr.A, clr.R, clr.G, clr.B);
            Logging.Write(LogLevel.Normal, newColor, log);
            lastlog = log;
        }

        public struct DoubleCastSpell
        {
            public DoubleCastSpell(string spellName, double expiryTime, DateTime currentTime)
                : this()
            {
                DoubleCastSpellName = spellName;
                DoubleCastExpiryTime = expiryTime;
                DoubleCastCurrentTime = currentTime;
            }

            private string DoubleCastSpellName { [UsedImplicitly] get; set; }

            public double DoubleCastExpiryTime { get; set; }

            public DateTime DoubleCastCurrentTime { get; set; }
        }
    }
}