﻿#region

using System;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net;
using System.Text.RegularExpressions;
using System.Xml;
using Styx.Common;

#endregion

namespace Superbad
{
    internal class Updater
    {
        private const string SvnURL = "http://superbad.googlecode.com/svn/trunk/";
        private const string ChangeLogURL = "http://code.google.com/p/superbad/source/detail?r=";

        private static readonly Regex LinkPattern = new Regex(@"<li><a href="".+"">(?<ln>.+(?:..))</a></li>",
                                                              RegexOptions.CultureInvariant);

        public static string changeLog;
        public static string newString;

        private static readonly Regex ChangelogPattern =
            new Regex(
                "<h4 style=\"margin-top:0\">Log message</h4>\r?\n?<pre class=\"wrap\" style=\"margin-left:1em\">(?<log>.+\r?\n?.+\r?\n?.+\r?\n?.+\r?\n?.+\r?\n?.+\r?\n?.+\r?\n?.+\r?\n?.+\r?\n?.+\r?\n?.+\r?\n?.+\r?\n?)</pre>",
                RegexOptions.CultureInvariant);

        public static int CCRevision
        {
            get
            {
                var revision = 0;

                try
                {
                    var path = Path.Combine(Path.GetDirectoryName(Process.GetCurrentProcess().MainModule.FileName),
                                            @"Routines\Superbad\MiscStuff.xml");

                    var reader = new FileStream(path, FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
                    var xmlDocument = new XmlDocument();
                    xmlDocument.Load(reader);
                    var nodeList = xmlDocument.GetElementsByTagName("MiscInformation");
                    revision = Convert.ToInt16(nodeList[0].FirstChild.ChildNodes[0].InnerText);
                }

                catch
                {
                }

                return revision;
            }

            set
            {
                var path = Path.Combine(Path.GetDirectoryName(Process.GetCurrentProcess().MainModule.FileName),
                                        @"Routines\Superbad\MiscStuff.xml");

                var reader = new FileStream(path, FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
                var xmlDocument = new XmlDocument();
                xmlDocument.Load(reader);

                var nodeList = xmlDocument.GetElementsByTagName("MiscInformation");
                nodeList[0].FirstChild.ChildNodes[0].InnerText = value.ToString();

                var writer = new FileStream(path, FileMode.Open, FileAccess.Write, FileShare.ReadWrite);
                xmlDocument.Save(writer);
            }
        }

        public static void CheckForUpdate()
        {
            try
            {
                var revision = CCRevision;
                var onlineRevision = GetOnlineRevision();

                Logging.Write("Checking online for new revision of Superbad");
                if (revision < onlineRevision)
                {
                    //string changeLog = GetChangeLog(onlineRevision);
                    Logging.Write(
                        string.Format("Revision {0} is available for download, you are currently using rev {1}.",
                                      onlineRevision, revision));
                    Logging.Write("This will now download in the background, you will be informed when its complete.");

                    DownloadFilesFromSvn(new WebClient(), SvnURL);
                    Logging.Write(" ");
                    Logging.Write("Download of revision " + onlineRevision +
                                  " is complete. You must close and restart HB for the changes to be applied.");
                    Logging.Write(" ");
                    CCRevision = onlineRevision;
                    changeLog = GetChangeLog(onlineRevision);
                    newString = changeLog.Replace("//", Environment.NewLine);
                    new ChangelogForm().ShowDialog();
                }
                else
                {
                    Logging.Write("No updates have been found. Revision " + revision + " is the latest build.");
                }
            }
            catch
            {
            }
        }

        private static int GetOnlineRevision()
        {
            var client = new WebClient();
            var html = client.DownloadString(SvnURL);
            var pattern = new Regex(@" - Revision (?<rev>\d+):", RegexOptions.CultureInvariant);
            var match = pattern.Match(html);
            if (match.Success && match.Groups["rev"].Success) return int.Parse(match.Groups["rev"].Value);
            throw new Exception("Unable to retreive revision! The sky is falling!");
        }

        private static void DownloadFilesFromSvn(WebClient client, string url)
        {
            var basePath = Path.Combine(Path.GetDirectoryName(Process.GetCurrentProcess().MainModule.FileName),
                                        @"Routines\Superbad\");
            var html = client.DownloadString(url);
            var results = LinkPattern.Matches(html);

            var matches = from match in results.OfType<Match>()
                          where match.Success && match.Groups["ln"].Success
                          select match;
            foreach (var match in matches)
            {
                var file = RemoveXmlEscapes(match.Groups["ln"].Value);
                var newUrl = url + file;
                if (newUrl[newUrl.Length - 1] == '/') // it's a directory...
                {
                    DownloadFilesFromSvn(client, newUrl);
                }
                else // its a file.
                {
                    string filePath, dirPath;
                    if (url.Length > SvnURL.Length)
                    {
                        var relativePath = url.Substring(SvnURL.Length);
                        dirPath = Path.Combine(basePath, relativePath);
                        filePath = Path.Combine(dirPath, file);
                    }
                    else
                    {
                        dirPath = Environment.CurrentDirectory;
                        filePath = Path.Combine(basePath, file);
                    }
                    if (!Directory.Exists(dirPath)) Directory.CreateDirectory(dirPath);
                    client.DownloadFile(newUrl, filePath);
                }
            }
        }

        private static string RemoveXmlEscapes(string xml)
        {
            return
                xml.Replace("&amp;", "&").Replace("&lt;", "<").Replace("&gt;", ">").Replace("&quot;", "\"").Replace(
                    "&apos;", "'");
        }

        private static string GetChangeLog(int revision)
        {
            var client = new WebClient();
            var html = client.DownloadString(ChangeLogURL + revision);
            var match = ChangelogPattern.Match(html);
            if (match.Success && match.Groups["log"].Success) return RemoveXmlEscapes(match.Groups["log"].Value);
            return null;
        }
    }
}